/*
	PluginManager v4
	Copyright (C) 2011, plum

	This program is free software: you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation, either version 3 of the License, or
	(at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

// header
#include "plugin_mgr.h"

// function
int CheckEof(SceUID fd)
{
	char c;

	if(sceIoRead(fd, &c, 1) == 1)
	{
		sceIoLseek(fd, -1, PSP_SEEK_CUR);
		return 0;
	}

	return 1;
}

u32 ConvertKeys(const char *buf)
{
	if(strcmp(buf, "SELECT") == 0)			return PSP_CTRL_SELECT;
	else if(strcmp(buf, "START") == 0)		return PSP_CTRL_START;
	else if(strcmp(buf, "UP") == 0)			return PSP_CTRL_UP;
	else if(strcmp(buf, "RIGHT") == 0)		return PSP_CTRL_RIGHT;
	else if(strcmp(buf, "DOWN") == 0)		return PSP_CTRL_DOWN;
	else if(strcmp(buf, "LEFT") == 0)		return PSP_CTRL_LEFT;
	else if(strcmp(buf, "L_TRI") == 0)		return PSP_CTRL_LTRIGGER;
	else if(strcmp(buf, "R_TRI") == 0)		return PSP_CTRL_RTRIGGER;
	else if(strcmp(buf, "TRIANGLE") == 0)	return PSP_CTRL_TRIANGLE;
	else if(strcmp(buf, "CIRCLE") == 0)		return PSP_CTRL_CIRCLE;
	else if(strcmp(buf, "CROSS") == 0)		return PSP_CTRL_CROSS;
	else if(strcmp(buf, "SQUARE") == 0)		return PSP_CTRL_SQUARE;
	else if(strcmp(buf, "HOME") == 0)		return PSP_CTRL_HOME;
	else if(strcmp(buf, "HOLD") == 0)		return PSP_CTRL_HOLD;
	else if(strcmp(buf, "NOTE") == 0)		return PSP_CTRL_NOTE;
	else if(strcmp(buf, "SCREEN") == 0)		return PSP_CTRL_SCREEN;
	else if(strcmp(buf, "VOLUP") == 0)		return PSP_CTRL_VOLUP;
	else if(strcmp(buf, "VOLDOWN") == 0)	return PSP_CTRL_VOLDOWN;
	else return 0;
}

char *SplitString(char *str, const char *delim)
{
	char *ptr = strstr(str, delim);

	if(ptr == NULL)
		return NULL;

	*ptr = '\0';
	return ptr + strlen(delim);
}

u32 GetKeys(const char *str)
{
	u32 key;
	char buf[256];
	char *ptr;

	// set
	key = 0;
	strcpy(buf, str);

	do
	{
		ptr = SplitString(buf, "+");
		key |= ConvertKeys(buf);

		if(ptr != NULL)
			strcpy(buf, ptr);
	}
	while (ptr != NULL);

	return key;
}

u32 GetConfig(char *path)
{
	SceUID fd;
	char buf[256];
	char *ptr;
	u32 buttons;

	// set default buttons
	buttons = (PSP_CTRL_LTRIGGER | PSP_CTRL_RTRIGGER | PSP_CTRL_START);

	// make path
	ptr = strrchr(path, '/');

	if(!ptr)
		return buttons;

	strcpy(ptr + 1, "plugin_mgr.ini");

	// open
	fd = sceIoOpen(path, PSP_O_RDONLY, 0777);

	if(fd < 0)
		return buttons;

	while(CheckEof(fd) == 0)
	{
		ReadLine(fd, buf, 255);

		if(buf[0] != '#' && buf[0] != '\0')
		{
			ptr = SplitString(buf, "=");

			if(ptr == NULL)
				continue;

			if(strcmp(buf, "MENU_KEY") == 0)
			{
				buttons = GetKeys(ptr);
				break;
			}
		}
	}

	sceIoClose(fd);
	return buttons;
}

