/*
	PluginManager v4
	Copyright (C) 2011, plum

	This program is free software: you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation, either version 3 of the License, or
	(at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#ifndef __PLUGIN_MANAGER_H__
#define __PLUGIN_MANAGER_H__

// header
#include <pspkernel.h>
#include <systemctrl.h>
#include <pspdisplay.h>
#include <pspinit.h>
#include <pspctrl.h>
#include <string.h>
#include <stdarg.h>

// define
#define MAKE_JUMP(a, f) _sw(0x08000000 | (((u32)(f) & 0x0FFFFFFC) >> 2), a);
#define REDIRECT_FUNCTION(a, f) _sw(0x08000000 | (((u32)(f) >> 2) & 0x03FFFFFF), a); _sw(0, a + 4);

// struct
typedef struct SceModuleContainer
{
	int patch_flag;
	int number;
	int stat;
	char *prxname;
	SceUID thid[10];
	int thid_count;
	SceModule2 mod;
	struct SceModuleContainer *next;
	struct SceModuleContainer *prev;
} SceModuleContainer;

typedef struct BlackList
{
	char *path;
	struct BlackList *next;
} BlackList;

typedef struct
{
	char *path;
	int stat;
} PluginList;

typedef struct
{
	int width;
	int height;
	int bufferwidth;
	int pixelformat;
	void *vram;
} DrawBuffer;

// main.c
void ClearCaches(void);

// modpatch.c
void AdjustThreads(SceModuleContainer *top);
void DeleteThreadId(SceModuleContainer *top);
void PatchModuleMgr(void);
void PatchThreadStub(SceModule2 *mod, SceModuleContainer *ptr);

// utils.c
void *malloc_p(SceSize size);
int free_p(void *ptr);
int ReadLine(SceUID fd, char *buf, int size);
int SetContainer(const char *prxpath, SceModule2 *module, int count);
void AdjustContainer(SceModuleContainer *top, int flag);
int SetBlackList(const char *prxpath);
SceUID LoadStartModule(const char *path);
SceUID CreateStartThread(const char *name, SceKernelThreadEntry entry, int pri, int stack_size);
int InitThreadList(void);
void SuspendThreads(void);
void ResumeThreads(void);

// menu.c
int PluginMgrThread(SceSize args, void *argp);

// draw.c
void GetDrawInfo(void);
void ClearDisplay(void);
int BlitPrint(int x, int y, u32 fg_col, u32 bg_col, const char *str);
int BlitPrintf(int x, int y, u32 fg_col, u32 bg_col, const char *format, ...);
int BlitPrintfCenter(int y, u32 fg_col, u32 bg_col, const char *format, ...);
int BlitPrintfRight(int y, u32 fg_col, u32 bg_col, const char *format, ...);
int PutLine(int y, u32 fg_col, u32 bg_col, char ch);
void DrawLine(int y, u32 color);

// conf.c
int CheckEof(SceUID fd);
u32 GetConfig(char *path);

// printf.c
int vsnprintf(char *buf, int n, const char *format, va_list ap);
int snprintf(char *buf, int n, const char *format, ...);

// log.c
int Log_printf(const char *format, ...);

#endif

